package RNA::AntennaTree;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# アンテナのツリー構造
#	XML::RSS (RSS 1.0) の構造に似せてある
#
#
#         アンテナ             サイト               コンテンツ（一次）
# root ┬ {channel}
#      └ {items}   ┬ [0]  ┬ {channel}
#                   ├ [1]  └ {items}   ┬ [0]  ┬ {channel}
#                                        ├ [1]  └ {items}   ┬ [0]
#                                                             ├ [1]
#
#
#
#
#==============================================================================


use strict;
use vars qw($VERSION);

$VERSION = '0.1';

my $all_category = '__ALL__';

my $modules = {
	'http://purl.org/rss/1.0/modules/syndication/' => 'syn',
	'http://purl.org/dc/elements/1.1/' => 'dc',
	'http://webns.net/mvcb/' => 'admin',
	'http://purl.org/dc/terms/' => 'dcterms',
	'http://madskills.com/public/xml/rss/module/trackback/' => 'trackback',
	'http://www.semblog.org/ns/semblog/0.1/' => 'semblog'

};
#    'http://purl.org/rss/1.0/modules/taxonomy/' => 'taxo',

my %v1_0_ok_fields = (
    channel => {
		title       => '',
		description => '',
		link        => '',
		},
    image  => {
		title => '',
		url   => '',
		link  => ''
		},
    textinput => {
		title       => '',
		description => '',
		name        => '',
		link        => ''
		},
    skipDays  => {
		day         => ''
		},
    skipHours => {
		hour        => ''
		},
    items => [],
    num_items => 0,
    version         => '',
    encoding        => '',
    output          => '',
);

my %dc_ok_fields = (
	'title' => '',
	'creator' => '',
	'subject' => '',
	'description' => '',
	'publisher' => '',
	'contributor' => '',
	'date' => '',
	'type' => '',
	'format' => '',
	'identifier' => '',
	'source' => '',
	'language' => '',
	'relation' => '',
	'coverage' => '',
	'rights' => '',
);

#=============================================================================
# サブルーチン
#=============================================================================
sub new {
    my $class = shift;
	my $self = {};
    
    bless $self, $class;
    
    $self->_initialize(@_);
    
    return $self;
}

sub _initialize {
    my $self = shift;
    my %hash = @_;

    # internal hash
    $self->{_internal} = {};

    # init num of items to 0
    $self->{num_items} = 0;

    # initialize items
    $self->{items} = [];

    # modules
    $self->{modules} = $modules;

    # encoding
    (exists($hash{encoding}))
	? ($self->{encoding} = $hash{encoding})
	    : ($self->{encoding} = 'UTF-8');

    # RSS version 1.0
    #} elsif ($self->{version} eq '1.0') {
	foreach my $i (qw(channel image textinput)) {
	#foreach my $i (keys(%v1_0_ok_fields)) {
	    my %template=%{$v1_0_ok_fields{$i}};
	    $self->{$i} = \%template;
    }
}

# initialize items
sub clear_items
{
	my $self = shift;
	$self->{items} = [];
}

#---------------------------------------------
# 値の設定
#---------------------------------------------

# タイトルなどの設定
# $antenna->initialize($title, $link, $description, $date, $creator,$language);
sub initialize
{
    my $self = shift;
	my ($title, $link, $description, $date, $creator, $language) = @_;

	$self->{channel} = {
		title 		=> $title,
		link		=> $link,
		description	=> $description,
		dc => {
			date       => $date,
			creator    => $creator,
			language   => $language
		}
	};
}


# RSS の内容をもとに，
# AntennaTree の {items}->[$j] の形式に合致するノードを作成して返す
#
# $antenna_item = $antenna->make_node_from_rss($rss)
#	$rss は，コピー元
sub make_node_from_rss
{
	my $self = shift;
	my $rss = shift;

	my $new_item = {};  # これが，一つのRSSドキュメントを示す

	#---------------------------------
	# <channel> のコピー
	#---------------------------------

	# <link>, <description>, <title>
	$new_item->{channel} = {};
	foreach my $key (keys %{$self->{channel}}) {
		$new_item->{channel}->{$key} = $rss->{channel}->{$key};
	}

	# モジュール（Dublin Core など）
	foreach my $module (values %{$self->{modules}}){
		$new_item->{channel}->{$module} = {};
		foreach my $key (keys %{$rss->{channel}->{$module}}) {
			$new_item->{channel}->{$module}->{$key} 
				= $rss->{channel}->{$module}->{$key};
		}
	}		
	#---------------------------------
	# <items> のコピー
	#---------------------------------
# debug
#	print $#{$rss->{items}} . "\n";
	$new_item->{items} = [];
	foreach my $i (0 .. $#{$rss->{items}}) {
		my $item_to = $self->make_node_from_item($rss->{items}->[$i]);
		push(@{$new_item->{items}}, $item_to);
	}

	return $new_item;
#	print %{$new_item->{items}->[0]->{channel}};
}


# RSS の <item> （RSS->{items}->[$i]） の内容をもとに，
# AntennaTree の {items}->[$j] の形式に合致するノードを作成して返す
#
# $antenna_item = $antenna->make_node_from_item($item)
#	$item は，コピー元 RSS->{items}->[$i]

sub make_node_from_item
{
	my $self = shift;
	my $item_from = shift;
	my $item_to = {};
	
	# <link>, <description>, <title>
	$item_to->{channel} = {};
	foreach my $key (keys %{$self->{channel}}) {
		$item_to->{channel}->{$key} = $item_from->{$key};
	}
	# モジュール（Dublin Core など）
	foreach my $module (values %{$self->{modules}}){
		$item_to->{channel}->{$module} = {};
		foreach my $key (keys %{$item_from->{$module}}) {
			$item_to->{channel}->{$module}->{$key} 
				= $item_from->{$module}->{$key};
		}
	}

	return $item_to;
}


# RSS の配列へのリファレンスから，値をセット
#
# $antenna->set_from_rss_array($rss_array_ref)
sub set_from_rss_array
{
    my $self = shift;
	my $rss_array_ref = shift;
    $self->clear_items();

	# 各 $rss についての処理
	foreach my $rss (@{$rss_array_ref}){
		my $new_item = $self->make_node_from_rss($rss);
		push(@{$self->{items}}, $new_item);
# debug
#		print %{$self->{items}->[-1]->{items}->[0]->{channel}};
#		print "\n";
	}
}


# RSS の <item> の配列へのリファレンスから，値をセット
#
# $antenna->set_from_items_array($item_array_ref)
sub set_from_items_array
{
	my $self = shift;
	$self->clear_items();
	my $item_array_ref = shift;

	foreach my $i (0 .. $#{$item_array_ref}) {
		my $item_to = $self->make_node_from_item($item_array_ref->[$i]);
		push(@{$self->{items}}, $item_to);
	}
}

#---------------------------------------------
# Trackback，リンク 情報の付加
#---------------------------------------------

# $self->add_trackback_info($hash_ref)
#
# 	$hash_ref = {
#		'http://****/***.html' => $rss,
#		'http://...' => $rss2,
#		...
#	}
sub add_trackback_info
{
	my $self = shift;
	my $hash_ref = shift;

	$self->__add_trackback_info($hash_ref, $self);

#print STDERR $self->{items}->[29]->{channel}->{link} . "\n";
#print STDERR $self->{items}->[29]->{items}->[0]->{channel}->{link} . "\n";
}


# $self->__add_trackback_info($hash_ref, $node);
sub __add_trackback_info
{
	my $self = shift;
	my ($hash_ref, $node) = @_;
	my $uri = $node->{channel}->{link};

	# 対象ノードの下のノードを処理
	foreach my $item (@{$node->{items}}) {
		$self->__add_trackback_info($hash_ref, $item);
	}

	# 対象ノードの Trackback 情報を取得
	if($hash_ref->{$uri}) {
		my $rss = $hash_ref->{$uri};

		$node->{items} || ($node->{items} = []);
		foreach my $src_item (@{$rss->{items}}) {
			push(@{$node->{items}}, $self->make_node_from_item($src_item));
		}
	}
}


# $self->is_trackback($t_item)
sub is_trackback
{
	my $self = shift;
	my $node = shift;

	return RNA::RSS::is_trackback($node->{channel});
}

#---------------------------------------------
# CDATA
#---------------------------------------------

# $str_ref = $self->convert_cdata(\$str);
#
# <![CDATA[     ]]>
#
sub convert_cdata 
{
	my $self = shift;
	my $str_ref = shift;

	$$str_ref =~ s/\<\!\[CDATA\[(.*?)\]\]\>/$self->__convert_cdata($1)/ges;

	return $str_ref;
}

# $str = $self->__convert_cdata($cdata);
#
# <![CDATA[     ]]>
#
sub __convert_cdata 
{
	my $self = shift;
	my $str = shift;

	# <a href="uri">string</a>  → string [[uri]]  
#	$str =~ s/\<a.*?href\=["']?(\S+?)["']?(\s+?.+?)*\s*\>(.*?)\<\/a\>/$3 \[\[$1\]\] /gsi;

	# <img src="uri" /> → ( img [[uri]] )
#	$str =~ s/\<img.*?src\=["']?(\S+?)["']?(\s+?.+?)*\s*\/?\s*?\>/\( img \[\[$1\]\] \)/gsi;
	# remove tags
#	$str =~ s/\<.*?\>//gsi;

	# escape tag
	$str =~ s/\</&lt;/g;
	$str =~ s/\>/&gt;/g;
	$str =~ s/"/&quot;/g;

	return $str;
}

#---------------------------------------------
# テンプレート適用
#---------------------------------------------

# $str_ref = $antenna->apply_template($template_string, $hash_ref)
#
# 	$hash_ref = {
#		RNAStyleSheet => 'http://foo.ne.jp/style.css',
#		RNAJavaScript => 'http://foo.ne.jp/script.js'
#	}
#
sub apply_template
{
    my $self = shift;
	my $template = shift;
	my $hash_ref = shift;

	my $templ_ref = \$template;

	$templ_ref = $self->apply_template_channel($templ_ref, $hash_ref);

	# カテゴリ
	$templ_ref = $self->apply_template_categories($templ_ref, $self->{items});

	# サイト，エントリ
	$templ_ref = $self->apply_template_sites($templ_ref, $self->{items});
	$templ_ref = $self->apply_template_items($templ_ref, $self->{items});

	# CDATA を処理
	$templ_ref = $self->convert_cdata($templ_ref);

	return $templ_ref;
}

# $modified_templ_ref = $antenna->apply_template_channel($templ_ref, $hash_ref)
sub apply_template_channel
{
    my $self = shift;
	my ($templ_ref, $hash_ref) = @_;
	my $template = $$templ_ref;		# neccesary
	
	my %text_hash = %{$hash_ref};
	$text_hash{RNAPageTitle}       = $self->{channel}->{'title'};
	$text_hash{RNAPageDescription} = $self->{channel}->{'description'};
	$text_hash{RNAPageDate}        = $self->{channel}->{'dc'}->{'date'};
	$text_hash{RNALanguage}        = $self->{channel}->{'dc'}->{'language'};
	
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	return \$template;
}


# $modified_templ_ref = $antenna->apply_template_categories($templ_ref, $items_ref)
#
# 	<RNACategories></RNACategories>
#
sub apply_template_categories
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $template = $$templ_ref;		# neccesary

	$template =~ s/(\<RNACategories\>.*?\<\/RNACategories\>)/${$self->__apply_template_categories(\$1, $items_ref)}/ges;

	return \$template;
}


# $modified_templ_ref = $antenna->__apply_template_categories($templ_ref, $items_ref)
#
sub __apply_template_categories
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $template = $$templ_ref;		# neccesary

	# <RNACategories> の削除
	$template =~ s/\<\/?RNACategories\>//gs;

	my $ret_str = '';

	#-----------------------
	# カテゴリ一覧
	#-----------------------
	my %temp_hash = ();
	foreach my $item (@{$items_ref}){
		$temp_hash{$item->{channel}->{dc}->{subject}} = 1;
	}
	my @category_list = sort keys %temp_hash;
	
	foreach my $cat (@category_list) {
		my $t = $template;

		my %text_hash = (
			RNACategoryName => $cat
		);

		foreach my $s (keys %text_hash){
			$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
		}

		$t =~ s/(\<RNASites\>.*?\<\/RNASites\>)/${$self->__apply_template_sites(\$1, $items_ref, $cat)}/ges;

		$t =~ s/(\<RNAItems\>.*?\<\/RNAItems\>)/${$self->__apply_template_items(\$1, $items_ref, $cat)}/ges;

		$ret_str .= $t;
	}

	return \$ret_str;
}



# $modified_templ_ref = $antenna->apply_template_sites($templ_ref, $items_ref)
#
# 	<RNASites></RNASites>
#
sub apply_template_sites
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $template = $$templ_ref;		# neccesary

	$template =~ s/(\<RNASites\>.*?\<\/RNASites\>)/${$self->__apply_template_sites(\$1, $items_ref, $all_category)}/ges;

	return \$template;
}


# $modified_templ_ref = $antenna->__apply_template_sites($templ_ref, $items_ref, $category)
#
sub __apply_template_sites
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $category = shift;
	my $template  = $$templ_ref;		# neccesary

	# <RNASites> の削除
	$template =~ s/\<\/?RNASites\>//gs;

	my $ret_str = '';
	foreach my $item (@{$items_ref}) {
		# カテゴリが異なる場合は，次に進む
		if($category ne $all_category 
			&& $category ne $item->{channel}->{dc}->{subject}){
			next;
		}
		
		my $t = $template;

		my %text_hash = (
			RNASiteTitle => $item->{channel}->{title},
			RNASiteLink => $item->{channel}->{link},
			RNASiteDescription => $item->{channel}->{description},
			RNASiteCreator => $item->{channel}->{dc}->{creator},
			RNASiteDate => $item->{channel}->{dc}->{date}
		);

		foreach my $s (keys %text_hash){
			$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
		}

		my $contents = $item->{items};
		$t =~ s/(\<RNAContents\>.*?\<\/RNAContents\>)/${$self->__apply_template_contents(\$1, $contents)}/ges;

		$ret_str .= $t;
	}

	return \$ret_str;
}


# $modified_templ_ref = &apply_template_contents($templ_ref, $contents_ref)
#
# 	<RNAContents>.*?<\/RNAContents>
#
sub __apply_template_contents
{
    my $self = shift;
	my $templ_ref = shift;
	my $contents_ref = shift;
	my $template = $$templ_ref;		# neccesary

	# <RNAContents> の削除
	$template =~ s/\<\/?RNAContents\>//gs;

	my $ret_str = '';
	foreach my $content (@{$contents_ref}) {
		my $t = $template;

		my %text_hash = (
			RNAContentTitle => $content->{channel}->{title},
			RNAContentLink => $content->{channel}->{link},
			RNAContentDate => $content->{channel}->{dc}->{date},
			RNAContentDescription => $content->{channel}->{description},
			RNAContentSourceTitle => $content->{channel}->{dc}->{source},
			RNAContentSourceLink => $content->{channel}->{dcterms}->{isPartOf}
		);

		foreach my $s (keys %text_hash){
			$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
		}

		# Trackback, Hyper Link
		my $el = 'RNAIfTrackbacksExist';
		$t =~ s/(\<$el\>.*?\<\/$el\>)/${$self->__apply_template_if_trackbacks(\$1, $content)}/ges;

		$ret_str .= $t;
	}

	return \$ret_str;
}


# $modified_templ_ref = $antenna->apply_template_items($templ_ref, $items_ref)
#
# 	<RNAItems></RNAItems>
#
sub apply_template_items
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $template = $$templ_ref;		# neccesary

	$template =~ s/(\<RNAItems\>.*?\<\/RNAItems\>)/${$self->__apply_template_items(\$1, $items_ref, $all_category)}/ges;

	return \$template;
}


# $modified_templ_ref = $antenna->__apply_template_items($templ_ref, $items_ref, $category)
#
sub __apply_template_items
{
    my $self = shift;
	my $templ_ref = shift;
	my $items_ref = shift;
	my $category = shift;
	my $template = $$templ_ref;		# neccesary

	# <RNAItems> の削除
	$template =~ s/\<\/?RNAItems\>//gs;

	my $ret_str = '';
	foreach my $item (@{$items_ref}) {
		# カテゴリが異なる場合は，次に進む
		if($category ne $all_category 
			&& $category ne $item->{channel}->{dc}->{subject}){
			next;
		}

		my $t = $template;

		my %text_hash = (
			RNAItemTitle => $item->{channel}->{title},
			RNAItemLink => $item->{channel}->{link},
			RNAItemDescription => $item->{channel}->{description},
			RNAItemCreator => $item->{channel}->{dc}->{creator},
			RNAItemDate => $item->{channel}->{dc}->{date},
			RNAItemSourceTitle => $item->{channel}->{dc}->{source},
			RNAItemSourceLink => $item->{channel}->{dcterms}->{isPartOf}
		);

		foreach my $s (keys %text_hash){
			$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
		}

		# Trackback, Hyper Link
		my $el = 'RNAIfTrackbacksExist';
		$t =~ s/(\<$el\>.*?\<\/$el\>)/${$self->__apply_template_if_trackbacks(\$1, $item)}/ges;

		$ret_str .= $t;
	}

	return \$ret_str;
}

# Trackback, Hyper Link を追跡
#
#$self->__apply_template_if_trackbacks(\$1, $item)
#
# <RNAIfTrackbacksExist>.*?</RNAIfTrackbacksExist>
#
sub __apply_template_if_trackbacks
{
    my $self = shift;
	my $templ_ref = shift;
	my $item = shift;
	my $template = $$templ_ref;		# neccesary

	# <RNAIfTrackbacksExist> の削除
	$template =~ s/\<\/?RNAIfTrackbacksExist\>//gs;

	my $trackback_items_ref = $item->{items};

	# Trackback が無ければ，空文字列を返す
	unless ($self->is_trackback($trackback_items_ref->[0])) {
		my $s = '';
		return \$s;
	}
	
	$template =~ s/(\<RNATrackbacks\>.*?\<\/RNATrackbacks\>)/${$self->__apply_template_trackbacks(\$1, $trackback_items_ref)}/ges;

	return \$template;
}

# $templ_ref = $self->__apply_template_trackbacks($templ_ref, $trackback_items_ref)
#
# 	<RNATrackbacks>.*?<\/RNATrackbacks>
#
sub __apply_template_trackbacks
{
    my $self = shift;
	my $templ_ref = shift;
	my $trackback_items_ref = shift;
	my $template = $$templ_ref;		# neccesary

	# <RNATrackbacks> の削除
	$template =~ s/\<\/?RNATrackbacks\>//gs;

	my $ret_str = '';
	foreach my $t_item (@{$trackback_items_ref}) {
		if($self->is_trackback($t_item)) {
			my $t = $template;

			my %text_hash = (
				RNATrackbackTitle       => $t_item->{channel}->{title},
				RNATrackbackURI         => $t_item->{channel}->{link},
				RNATrackbackDescription => $t_item->{channel}->{description}
			);

			foreach my $s (keys %text_hash){
				$t =~ s/\<\$$s\$\>/$text_hash{$s}/ges;
			}

			$ret_str .= $t;
		}
	}

	return \$ret_str;
}


#=============================================================================
1;

